from state_manager.json import JSONState
from prometheus_exporter.datastore import MetricsDataStoreManager
from prometheus_client.exposition import default_handler, CONTENT_TYPE_LATEST
import geoip2.database
import psutil
import time
import os


class EventState(JSONState):
    pass


class MetricState():
    def __init__(self, name='default', path=None, rotate=1):
        self.name = name
        self.path = path
        if not os.path.exists(path):
            os.makedirs(path, 0o700)
        self.mm = MetricsDataStoreManager()

    def push(self, endpoint='localhost', timeout=10, debug=False):
        try:
            default_handler(
                url=endpoint + '/api/v1/import/prometheus',
                method="POST",
                timeout=timeout,
                headers=[("Content-Type", CONTENT_TYPE_LATEST)],
                data=self.mm.dump_prometheus_metrics(),
            )()
        except Exception as e:
            raise Exception("failed to push metrics %s" % e)

    def dump(self, debug=False):
        self.mm.serialize(self.mm, os.path.join(
            self.path, self.name + '.json'))
        if debug:
            print(self.mm.dump_prometheus_metrics().decode("utf-8"))

    def gauge(self, metric=None, tags=[], tag_values=[], value=None):
        try:
            self.mm.get_or_create_store(
                metric, 'Gauge', tags)
            getattr(self.mm, metric).get_or_create(
                (tag_values)).set(value)
        except Exception as err:
            self.log.error("failed to create gauge: %s" % err)

    def counter(self, metric=None, tags=[], tag_values=[], step=1):
        try:
            self.mm.get_or_create_store(
                metric, 'Counter', tags)

            getattr(self.mm, metric).get_or_create(
                (tag_values)).incr(step=step)
        except Exception as err:
            self.log.info("failed to get metric store")


class Timer():
    def __enter__(self):
        self.start = time.time()
        return self

    def __exit__(self, *args):
        self.end = time.time()
        self.seconds = round(self.end - self.start, 3)


def load_too_high(max_load=40):
    load_5 = psutil.getloadavg()[1]
    if load_5 >= max_load:
        return True


def geo_lookup_ip(ip):
    path = os.path.dirname(os.path.abspath(__file__))
    try:
        with geoip2.database.Reader(path + '/../data_files/GeoLite2-Country.mmdb') as reader:
            response = reader.country(ip)
            if response:
                return response.country.iso_code
    except geoip2.errors.AddressNotFoundError:
        pass
