from dataclasses import fields
from prometheus_client import CollectorRegistry, Counter, Gauge, generate_latest
from prometheus_client.exposition import default_handler, CONTENT_TYPE_LATEST


def push(data, endpoint, prefix='', labels={}, timeout=5, debug=False):
    reg = CollectorRegistry()
    for f in fields(data):
        prom_obj = f.metadata.get('type')
        if prom_obj:
            label_config = f.metadata.get('labels', [])
            set_labels = {k: v for k, v in labels.items() if k in label_config}
            if len(set_labels) == len(label_config):
                j = prom_obj('_'.join((prefix, f.name)), '',
                             labelnames=label_config, registry=reg)
                j = j.labels(**set_labels)
                if isinstance(j, Counter):
                    j.inc(getattr(data, f.name))
                if isinstance(j, Gauge):
                    j.set(getattr(data, f.name))
            else:
                raise Exception("missing labels : %s" %
                                ([l for l in label_config if l not in labels]))

            if debug:
                print(generate_latest(reg).decode('utf-8'))
            else:
                try:
                    default_handler(
                        url=endpoint + '/api/v1/import/prometheus',
                        method="POST",
                        timeout=timeout,
                        headers=[("Content-Type", CONTENT_TYPE_LATEST)],
                        data=generate_latest(reg),
                    )()
                except Exception as e:
                    raise Exception("failed to push metrics %s" % e)
